//
// MoonCanvas.cpp
// Draw the moon and related info.
//
//    Copyright 2002 by Akkana Peck.
// You may use or distribute this code under the terms of the GPL.
//

#include "MoonCanvas.h"

#include "MoonFeature.h"

#include <qpainter.h>
#include <qimage.h>
#include <qmessagebox.h>

// XXX next two should be removable:
#include <stdio.h>
#include <math.h>
#include <stdlib.h>

MoonCanvas::MoonCanvas( QWidget *parent, const char *name )
    : SkyCanvas( parent, name ),
      moonRadius(0), moonCenterX(-1), moonCenterY(-1), mMirrored(1)
{
}

MoonCanvas::~MoonCanvas()
{
    if (mPainter)
        delete mPainter;
    if (moonImg)
        delete moonImg;
}

void MoonCanvas::Init()
{
    moonImg = new QImage (QString("/opt/Qtopia/share/qmoon/fullmoon.png"));
    mImgLibrationE = 4;
    mImgLibrationN = 1;

    setMinimumWidth(moonImg->width());
    setMinimumHeight(moonImg->height());

    moonCenterX = width()/2;
    moonCenterY = moonCenterX;

    moonRadius = moonImg->width()/2 - 4;

    mPainter = new QPainter(this);

    MoonFeature::InitMoonFeatures(moonRadius);

    MoonFeature::SetCanvasSize(width(), height());
}

void MoonCanvas::ToggleMirrored()
{
    mMirrored = -mMirrored;

    moonImg = new QImage (QString(mMirrored > 0
                                ? "/opt/Qtopia/share/qmoon/fullmoon.png"
                                : "/opt/Qtopia/share/qmoon/fullmoon-mirror.png"
                              ));
    moonRadius = moonImg->width()/2 - 4;
    Paint();
}

QSize MoonCanvas::minimumSizeHint() const
{
    return QSize(width(), width());
}

void MoonCanvas::Paint()
{
    if (moonRadius <= 0)
        Init();
    if (!mPainter)
        return;

    Clear();
    mPainter->drawImage(moonCenterX-moonImg->width()/2,
                        moonCenterY-moonImg->height()/2,
                        *moonImg);
    int i;
    for (i=0; i < mCraterNum; ++i)
        mMatches[i]->PaintFeature(mPainter, moonCenterX, moonCenterY,
                                  moonRadius,
                                  mMirrored, mImgLibrationE, mImgLibrationN);

    return;
}

void MoonCanvas::PaintFeature(MoonFeature* feature)
{
    if (mCraterNum >= MAX_OBJECTS_SHOWN-1)
        return;

    mMatches[mCraterNum++] = feature;

    feature->PaintFeature(mPainter, moonCenterX, moonCenterY, moonRadius,
                          mMirrored, mImgLibrationE, mImgLibrationN);

    mDescriptions.append(feature->GetName());
    QString numstr;
    numstr.setNum(feature->GetSize());
    mDescriptions.append(" (");
    mDescriptions.append(numstr);
    mDescriptions.append(" km)\n");
    mDescriptions.append(feature->GetDesc());
    mDescriptions.append("\n");
}

void MoonCanvas::mousePressEvent(QMouseEvent *event)
{
    // Remember the time:
    gettimeofday(&mLastButtonDownTime, 0);

    // We're only interested in doubleclick events:
    if (event->type() != QEvent::MouseButtonDblClick) {
        mCraterNum = 0;
        mDescriptions.truncate(0);
    }
    else
        QMessageBox::information(this, "QMoon", mDescriptions,
                                 QMessageBox::Ok);
}

void MoonCanvas::mouseReleaseEvent(QMouseEvent *event)
{
    // Get the time of the event -- we'll use it later:
    struct timeval buttonUpTime;
    gettimeofday(&buttonUpTime, 0);

    Paint();    // clear the canvas

    double fx = (event->pos().x() - moonCenterX) * mMirrored;
    double fy = event->pos().y() - moonCenterY;

    // Translate that to moon coordinates
    double fLatitude = -asin(fy / moonRadius);
    double latitude = fLatitude * 180./M_PI - mImgLibrationN;
    double longitude = asin(fx / moonRadius / cos(fLatitude)) * 180./M_PI
                        - mImgLibrationE;

    // Figure out which crater lives there
    double mindist = moonRadius*moonRadius;
    MoonFeature* nearfeature = 0;
    MoonFeature* feature;
    MoonFeature::InitLoop();
    while ((feature = MoonFeature::NextFeature()) != 0)
    {
        int isInside;
        double thisdist = feature->Distance(longitude, latitude, &isInside);
        if (isInside)
        {
            // For big maria, lots of things will be inside
            // and it can be hard to click exactly on an object.
            // So for big features, don't let "inside" stop us
            // from looking for a nearby crater.
            if (feature->GetSize() < 150)
            {
                mindist = 0;
                nearfeature = 0;
            }
            PaintFeature(feature);
            //printf("Inside %s\n", feature->GetName());
        }
        else if (thisdist < mindist)
        {
            nearfeature = feature;
            mindist = thisdist;
        }
    }

    // Draw the nearby crater if we aren't inside of one:
    if (nearfeature)
    {
        //printf("Landed near %s\n", nearfeature->GetName());
        PaintFeature(nearfeature);
    }

    // If the user held the mouse down for a while,
    // then pop up an information dialog as well:
    int dusec = 1000000 * (buttonUpTime.tv_sec - mLastButtonDownTime.tv_sec)
        + buttonUpTime.tv_usec - mLastButtonDownTime.tv_usec;
    if (dusec > 850000)
        QMessageBox::information(this, "QMoon", mDescriptions,
                                 QMessageBox::Ok);
}

#if 0
QSizePolicy MoonCanvas::sizePolicy() const
{
    //return QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Fixed );
    QSizePolicy policy(QSizePolicy::Expanding, QSizePolicy::Fixed);
    policy.
    return policy;
}
#endif
